import sys
import matplotlib.pyplot as plt
# ParaMol imports
from ParaMol.System.system import *
from ParaMol.MM_engines.openmm import *
from ParaMol.QM_engines.qm_engine import *

# ParaMol Tasks imports
from ParaMol.Tasks.parametrization import *
from ParaMol.Tasks.torsions_parametrization import *
from ParaMol.Utils.settings import *
from ParaMol.Utils.Symmetrizers.amber_symmetrizer import *

# --------------------------------------------------------- #
#                         Preparation                       #
# --------------------------------------------------------- #
# Create the OpenMM engine for aspirin
openmm_system = OpenMMEngine(init_openmm=True, topology_format='AMBER', top_file='aspirin.prmtop', crd_format="AMBER", crd_file='aspirin.inpcrd')

# Create ParaMol System
aspirin = ParaMolSystem(name="aspirin_boltzmann", engine=openmm_system, n_atoms=21)

# Create ParaMol's force field representation and ask to parametrize bonds, angles and torsions
aspirin.force_field.create_force_field(ff_file="aspirin_sym.ff")

# Create ParaMol settings instance
paramol_settings = Settings()

# The objective function will contain a energy, force and regularization term
paramol_settings.properties["include_energies"] = True
paramol_settings.properties["include_forces"] = False
paramol_settings.properties["include_regularization"] = True
paramol_settings.properties["regularization"]["scaling_factor"] = 1.0

# Objective function settinigs
paramol_settings.objective_function["weighing_method"] = "boltzmann"
paramol_settings.objective_function["weighing_temperature"] = 500.0 * unit.kelvin

# Read data
aspirin.read_data("aspirin_dftb_scan_totally_relaxed.nc")

systems = [aspirin]

# --------------------------------------------------------- #
#                  MM-relaxed conformations                 #
# --------------------------------------------------------- #
torsion_to_scan = [[1, 3, 4, 5]]

torsion_type_to_fit = [[[1, 3, 4, 5],[1, 3, 4, 9]]]

# Restitute original energy with new conformations
aspirin.ref_coordinates = aspirin.ref_coordinates[:36]
aspirin.ref_energies = aspirin.ref_energies[:36]
aspirin.n_structures = len(aspirin.ref_energies)


n_points_scan = 36
torsions_scan = TorsionScan()

mm_relaxed = True

gaff_list = []

if mm_relaxed:
    # Save original data
    qm_energies = copy.deepcopy(aspirin.ref_energies)
    qm_coordinates = copy.deepcopy(aspirin.ref_coordinates)

    mm_relaxed_coordinates = []
    # --------------------------------------------------------- #
    #                  MM-relaxed optimizations                 #
    # --------------------------------------------------------- #
    for i in range(len(torsion_to_scan)):
        aspirin.ref_coordinates = qm_coordinates[i*n_points_scan:n_points_scan*(i+1)]
        aspirin.ref_energies = qm_energies[i*n_points_scan:n_points_scan*(i+1)]
        aspirin.n_structures = n_points_scan

        mm_relaxed_conf = torsions_scan.get_mm_relaxed_conformations(aspirin, [torsion_to_scan[i]], tolerance=0.01, max_iter=0, force_constant=9999999.0, threshold=1e-2)
        mm_relaxed_coordinates.append(mm_relaxed_conf)

        aspirin.ref_coordinates = mm_relaxed_conf

        # Parametrize the torsion
        aspirin.force_field.optimize_torsions_by_symmetry(torsion_type_to_fit[i], change_other_torsions=True, change_other_parameters=True, set_zero=True)

        gaff_list.append(aspirin.get_energies_ensemble())

        parametrization = Parametrization()
        systems, parameter_space, objective_function, optimizer = parametrization.run_task(paramol_settings, systems)


    # Restitute original energy with new conformations
    aspirin.ref_coordinates = np.concatenate(np.array(mm_relaxed_coordinates))
    qm_coordinates = copy.deepcopy(aspirin.ref_coordinates)
    aspirin.ref_energies = qm_energies
    aspirin.n_structures = len(aspirin.ref_energies)

    gaff_energies = np.concatenate(np.array(gaff_list))
    #gaff_energies = aspirin.get_energies_ensemble()


# --------------------------------------------------------- #
#                   Parametrize the Torsion                 #
# --------------------------------------------------------- #
mm_energies = aspirin.get_energies_ensemble()
aspirin.force_field.write_ff_file("aspirin_" + paramol_settings.objective_function["weighing_method"] + ".ff")

np.savetxt(paramol_settings.objective_function["weighing_method"] + "_gaff.dat", gaff_energies)
np.savetxt(paramol_settings.objective_function["weighing_method"] + "_mm.dat", mm_energies)
np.savetxt(paramol_settings.objective_function["weighing_method"] + "_qm.dat", qm_energies)

