import sys

# ParaMol imports
from ParaMol.System.system import *
from ParaMol.MM_engines.openmm import *
from ParaMol.QM_engines.qm_engine import *

# ParaMol Tasks imports
from ParaMol.Tasks.parametrization import *
from ParaMol.Tasks.torsions_parametrization import *
from ParaMol.Utils.settings import *
from ParaMol.Utils.Symmetrizers.amber_symmetrizer import *

# --------------------------------------------------------- #
#                         Preparation                       #
# --------------------------------------------------------- #
# Create the OpenMM engine for aspirin
openmm_system = OpenMMEngine(init_openmm=True, topology_format='AMBER', top_file='aspirin.prmtop', crd_format="AMBER", crd_file='aspirin_local_minimum.inpcrd')

# Create ParaMol System
aspirin = ParaMolSystem(name="aspirin", engine=openmm_system, n_atoms=21)

# Create ParaMol's force field representation and ask to parametrize bonds, angles and torsions
aspirin.force_field.create_force_field() #ff_file="aspirin_sym.ff")

# Create ParaMol settings instance
paramol_settings = Settings()

# The objective function will contain a energy, force and regularization term
paramol_settings.properties["include_energies"] = True
paramol_settings.properties["include_forces"] = False # One should not include forces when a torsional scan
paramol_settings.properties["include_regularization"] = True


paramol_settings.qm_engine["ase"]["opt_fmax"] = 1e-2

# --------------------------------------------------------- #
#                Symmetrize ParaMol ForceField              #
# --------------------------------------------------------- #
# Symmetry ParaMol ForceField so that it respects atom-type symmetries
amber_symmetrizer = AmberSymmetrizer(top_file="aspirin.prmtop")
amber_symmetrizer.get_symmetries()
amber_symmetrizer.symmetrize_force_field(aspirin.force_field)

# --------------------------------------------------------- #
#                     Set the QM Engine                     #
# --------------------------------------------------------- #
# Create the ASE calculator
from ase.calculators.dftb import *

calc = Dftb(Hamiltonian_='DFTB',  # line is included by default
	                Hamiltonian_MaxSCCIterations=1000,
			Hamiltonian_MaxAngularMomentum_='',
			Hamiltonian_MaxAngularMomentum_H='s',
			Hamiltonian_MaxAngularMomentum_O='p',
			Hamiltonian_MaxAngularMomentum_C='p',
			Hamiltonian_MaxAngularMomentum_N="p",
			Hamiltonian_Dispersion="DftD3 { \n s6=1.000 \n s8=0.5883 \n Damping = BeckeJohnson { \n a1=0.5719 \n a2=3.6017 \n } \n }",
			Hamiltonian_SCC='Yes',
			Hamiltonian_SCCTolerance=1e-8, )

# Alternative, we could set the calculator in the settings
paramol_settings.qm_engine["ase"]["calculator"] = calc
paramol_settings.qm_engine["ase"]["opt_log_file"] = "logopt.out"

# --------------------------------------------------------- #
#            Perform the Torsions Parametrization           #
# --------------------------------------------------------- #
torsions_parametrization = TorsionsParametrization()
torsions_parametrization.run_task(paramol_settings, [aspirin], scan_settings=[[-180.0,180.0,10.0]], optimize_mm=True, optimize_mm_type="FREEZE_ATOMS", optimize_qm_before_scan=True, parametrization_type="SIMULTANEOUS")

aspirin.write_data("aspirin_dftb_scan_totally_relaxed.nc")

# Update AMBER symmetrizer with new parameters
#amber_symmetrizer.update_term_types_parameters(parameter_space.optimizable_parameters)

# Write AMBER topology file (.prmtop) and .frcmod file
#amber_symmetrizer.save_prmtop("aspirin_opt.prmtop")
#amber_symmetrizer.save_frcmod("aspirin_opt.frcmod")
