import sys
sys.path.insert(0, "/home/treason/PycharmProjects/ParaMol_git_master")

# ParaMol imports
from ParaMol.System.system import *
from ParaMol.MM_engines.openmm import *
from ParaMol.Tasks.adaptive_parametrization import *
from ParaMol.Utils.settings import *
from ParaMol.Utils.Symmetrizers.amber_symmetrizer import *

# --------------------------------------------------------- #
#                         Preparation                       #
# --------------------------------------------------------- #
# Create the OpenMM engine for caffeine
openmm_system = OpenMMEngine(init_openmm=True, topology_format='AMBER', top_file='caffeine.prmtop', crd_format='AMBER', crd_file='caffeine.inpcrd')

# Create ParaMol System
caffeine = ParaMolSystem(name="caffeine", engine=openmm_system, n_atoms=24)

# Create ParaMol's force field representation and ask to parametrize bonds, angles and torsions
caffeine.force_field.create_force_field(opt_bonds=True, opt_angles=True, opt_torsions=True)

# --------------------------------------------------------- #
#                Symmetrize ParaMol ForceField              #
# --------------------------------------------------------- #
# Symmetry ParaMol ForceField so that it respects atom-type symmetries
amber_symmetrizer = AmberSymmetrizer(top_file="caffeine.prmtop")
amber_symmetrizer.get_symmetries()
amber_symmetrizer.symmetrize_force_field(caffeine.force_field)

# Write ParaMol Force Field file with final parameters
caffeine.force_field.write_ff_file("caffeine_sym.ff")

# Create ParaMol settings instance
paramol_settings = Settings()

# The objective function will contain a energy, force and regularization term
paramol_settings.properties["include_energies"] = True
paramol_settings.properties["include_forces"] = True
paramol_settings.properties["include_regularization"] = True

paramol_settings.properties["forces"]["weight"] = 0.5
paramol_settings.properties["energies"]["weight"] = 0.5


paramol_settings.properties["regularization"]["scaling_factor"] = 1.0
#paramol_settings.properties["regularization"]["scaling_factor"] = 1.0 / len(system.force_field.get_optimizable_parameters(symmetry_constrained=True)[1])

paramol_settings.parameter_space["parameters_magnitudes"] = {"charge": 0.2,
                                                             "lj_sigma": 0.20,
                                                             "lj_eps": 0.20,
                                                             "torsion_phase": np.pi,
                                                             "torsion_k": 4 * 4.184,
                                                             "bond_eq": 0.1,
                                                             "bond_k":  1e5,
                                                             "angle_eq": np.pi / 2,
                                                             "angle_k": 200.0,
                                                             "scee": 0.5,
                                                             "scnb": 0.5}


paramol_settings.parameter_space["scaling_constants_method"] = "arithmetic"
paramol_settings.parameter_space["prior_widths_method"] = "default"

# The QM engine used will be ParaMol's AMBER sqm wrapper
paramol_settings.qm_engine["qm_engine"] = "ase"


# --------------------------------------------------------- #
#                     Set the QM Engine                     #
# --------------------------------------------------------- #
# Create the ASE calculator
from ase.calculators.dftb import *

calc = Dftb(Hamiltonian_='DFTB',  # line is included by default
			Hamiltonian_MaxAngularMomentum_='',
			Hamiltonian_MaxAngularMomentum_H='s',
			Hamiltonian_MaxAngularMomentum_O='p',
			Hamiltonian_MaxAngularMomentum_C='p',
			Hamiltonian_MaxAngularMomentum_N="p",
			Hamiltonian_Dispersion="DftD3 { \n s6=1.000 \n s8=0.5883 \n Damping = BeckeJohnson { \n a1=0.5719 \n a2=3.6017 \n } \n }",
			Hamiltonian_SCC='Yes',
			Hamiltonian_SCCTolerance=1e-8, )

# Alternative, we could set the calculator in the settings
paramol_settings.qm_engine["ase"]["calculator"] = calc

# --------------------------------------------------------- #
#                  Adaptive Parametrization                 #
# --------------------------------------------------------- #
adaptive_parametrization = AdaptiveParametrization()
systems, parameter_space, objective_function, optimizer = adaptive_parametrization.run_task(paramol_settings, [caffeine], rmsd_tol=0.0001, max_iter=100, structures_per_iter=100, wham_reweighing=True)

# Update AMBER symmetrizer with new parameters
amber_symmetrizer.update_term_types_parameters(parameter_space.optimizable_parameters)

# Write AMBER topology file (.prmtop) and .frcmod file
amber_symmetrizer.save_prmtop("caffeine_opt.prmtop")
amber_symmetrizer.save_frcmod("caffeine_opt.frcmod")

# Write ParaMol Force Field file with final parameters
caffeine.force_field.write_ff_file("caffeine_adaptive.ff")
