import sys
sys.path.insert(0, "/home/jm4g18/ParaMol")

# ParaMol imports
from ParaMol.System.system import *
from ParaMol.MM_engines.openmm import *
from ParaMol.Tasks.adaptive_parametrization import *
from ParaMol.Utils.settings import *
from ParaMol.Utils.Symmetrizers.amber_symmetrizer import *

from os import environ

environ["ASE_ONETEP_COMMAND"] = "export OMP_NUM_THREADS=4; export I_MPI_PMI_LIBRARY=/usr/lib64/libpmi.so; export OMP_STACKSIZE=64M; mpirun -n 5 /mainfs/home/jm4g18/programs/onetep/bin/onetep.iridis5.intel18.omp.scalapack PREFIX.dat >> PREFIX.out 2> PREFIX.err"

# --------------------------------------------------------- #
#                         Preparation                       #
# --------------------------------------------------------- #
# Create the OpenMM engine for caffeine
openmm_system = OpenMMEngine(init_openmm=True, topology_format='AMBER', top_file='caffeine.prmtop', crd_format='AMBER', crd_file='caffeine.inpcrd')

# Create ParaMol System
caffeine = ParaMolSystem(name="caffeine", engine=openmm_system, n_atoms=24)

# Create ParaMol's force field representation and ask to parametrize bonds, angles and torsions
caffeine.force_field.create_force_field(opt_bonds=True, opt_angles=True, opt_torsions=True)

# --------------------------------------------------------- #
#                Symmetrize ParaMol ForceField              #
# --------------------------------------------------------- #
# Symmetry ParaMol ForceField so that it respects atom-type symmetries
amber_symmetrizer = AmberSymmetrizer(top_file="caffeine.prmtop")
amber_symmetrizer.get_symmetries()
amber_symmetrizer.symmetrize_force_field(caffeine.force_field)

# Write ParaMol Force Field file with final parameters
caffeine.force_field.write_ff_file("caffeine_sym.ff")

# Create ParaMol settings instance
paramol_settings = Settings()

# The objective function will contain a energy, force and regularization term
paramol_settings.properties["include_energies"] = True
paramol_settings.properties["include_forces"] = True
paramol_settings.properties["include_regularization"] = True

paramol_settings.properties["forces"]["weight"] = 0.5
paramol_settings.properties["energies"]["weight"] = 0.5

paramol_settings.properties["regularization"]["scaling_factor"] = 1.0 

paramol_settings.parameter_space["parameters_magnitudes"] = {"charge": 0.2,
                                                             "lj_sigma": 0.20,
                                                             "lj_eps": 0.20,
                                                             "torsion_phase": np.pi,
                                                             "torsion_k": 4 * 4.184,
                                                             "bond_eq": 0.1,
                                                             "bond_k":  1e5,
                                                             "angle_eq": np.pi / 2,
                                                             "angle_k": 200.0,
                                                             "scee": 0.5,
                                                             "scnb": 0.5}


paramol_settings.parameter_space["scaling_constants_method"] = "arithmetic"
paramol_settings.parameter_space["prior_widths_method"] = "default"

# The QM engine used will be ParaMol's AMBER sqm wrapper
paramol_settings.qm_engine["qm_engine"] = "ase"


# --------------------------------------------------------- #
#                     Set the QM Engine                     #
# --------------------------------------------------------- #
# Create the ASE calculator
from ase.calculators.onetep import *

calc = Onetep(label='caffeine')
calc.set(pseudo_path="/home/jm4g18/recpot")
calc.set(pseudo_suffix='-optgga1.recpot')
calc.set(task='SinglePoint', xc='VV10', cutoff_energy='700 eV')
calc.set(ngwf_threshold_orig="1e-4")
calc.set(ngwf_radius=8.0)

# Alternative, we could set the calculator in the settings
paramol_settings.qm_engine["ase"]["calculator"] = calc

# --------------------------------------------------------- #
#                  Adaptive Parametrization                 #
# --------------------------------------------------------- #
adaptive_parametrization = AdaptiveParametrization()
systems, parameter_space, objective_function, optimizer = adaptive_parametrization.run_task(paramol_settings, [caffeine], rmsd_tol=0.0001, max_iter=100, structures_per_iter=100, wham_reweighing=True, restart=True)

# Update AMBER symmetrizer with new parameters
amber_symmetrizer.update_term_types_parameters(parameter_space.optimizable_parameters)

# Write AMBER topology file (.prmtop) and .frcmod file
amber_symmetrizer.save_prmtop("caffeine_opt.prmtop")
amber_symmetrizer.save_frcmod("caffeine_opt.frcmod")

# Write ParaMol Force Field file with final parameters
caffeine.force_field.write_ff_file("caffeine_adaptive.ff")
