import numpy as np
import sys


# ParaMol imports
from ParaMol.System.system import *
from ParaMol.MM_engines.openmm import *
from ParaMol.QM_engines.qm_engine import *

# ParaMol Tasks imports
from ParaMol.Tasks.parametrization import *
from ParaMol.Tasks.torsions_scan import *
from ParaMol.Utils.settings import *
from ParaMol.Utils.Symmetrizers.amber_symmetrizer import *

# --------------------------------------------------------- #
#                         Preparation                       #
# --------------------------------------------------------- #
# Create the OpenMM engine for norfloxacin
prmtop_file="lig.prmtop"
openmm_system = OpenMMEngine(init_openmm=True, topology_format='AMBER', top_file=prmtop_file, crd_format='AMBER', crd_file='lig.inpcrd')

# Create ParaMol System
norfloxacin = ParaMolSystem(name="norfloxacin", engine=openmm_system, n_atoms=32)

# Create ParaMol's force field representation and ask to parametrize bonds, angles and torsions
norfloxacin.force_field.create_force_field(ff_file="norfloxacin_k.ff")

# Create ParaMol settings instance
paramol_settings = Settings()

# The objective function will contain a energy, force and regularization term
paramol_settings.properties["include_energies"] = True
paramol_settings.properties["include_forces"] = False # One should not include forces when a torsional scan
paramol_settings.properties["include_regularization"] = True
paramol_settings.properties["regularization"]["scaling_factor"] = 0.1
# Objective function settinigs
paramol_settings.objective_function["weighting_method"] = "uniform"
paramol_settings.objective_function["weighting_temperature"] = 500.0 * unit.kelvin

paramol_settings.parameter_space["scaling_constants_method"] = "arithmetic"
paramol_settings.parameter_space["prior_widths_method"] = "default"
paramol_settings.optimizer["scipy"]["options"] =  {"disp": True, "iprint": 2, "verbosity": True, "maxiter": 10000, "ftol": 1e-8, }
# --------------------------------------------------------- #
#                Symmetrize ParaMol ForceField              #
# --------------------------------------------------------- #
# Symmetry ParaMol ForceField so that it respects atom-type symmetries
amber_symmetrizer = AmberSymmetrizer(top_file="lig.prmtop")
amber_symmetrizer.get_symmetries(norfloxacin.force_field)
amber_symmetrizer.symmetrize_force_field(norfloxacin.force_field)

torsion_to_scan = [[5, 2, 9, 13]]
norfloxacin.read_data("norfloxacin_dftb_2d_scan.nc")

torsions_to_freeze = [[9, 6, 2, 5],[5, 2, 9, 13],]
# --------------------------------------------------------- #
#                   Parametrize the Torsion                 #
# --------------------------------------------------------- #
#norfloxacin.force_field.write_ff_file("norfloxacin_symm.ff")

torsions_scan = TorsionScan()
mm_relaxed_conf = torsions_scan.get_mm_relaxed_conformations(norfloxacin, torsions_to_freeze, tolerance=0.01, max_iter=0, force_constant=9999999.0, threshold=1.0)
norfloxacin.ref_coordinates = mm_relaxed_conf

gaff_energies = norfloxacin.get_energies_ensemble()
np.savetxt("gaff.dat", gaff_energies)

parametrization = Parametrization()
systems, parameter_space, objective_function, optimizer = parametrization.run_task(paramol_settings, [norfloxacin])

# Update AMBER symmetrizer with new parameters
amber_symmetrizer.update_term_types_parameters(parameter_space.optimizable_parameters)

# Write AMBER topology file .prmtop and .frcmod file
amber_symmetrizer.save("norfloxacin_opt.prmtop")
amber_symmetrizer.save_frcmod("norfloxacin_opt")

mm_energies = norfloxacin.get_energies_ensemble()
np.savetxt("uniform.dat", mm_energies)
np.savetxt("../norfloxacin_dftb_energies.dat",  norfloxacin.ref_energies)
